@@ -581,23 +581,35 @@ double getBrierScore(uint     obsSize,
                      double **outcomeCLS,
                      double  *denomCount,
                      double  *cpv);
-void getConditionalClassificationIndex(uint     size,
-                                       uint     rTarget,
-                                       double  *responsePtr,
-                                       double **outcomeCLS,
-                                       double  *maxVote,
-                                       double  *denomCount,
-                                       double  *cpv);
+void getConditionalClassificationIndexGrow(uint     size,
+                                           uint     rTarget,
+                                           double  *responsePtr,
+                                           double **outcomeCLS,
+                                           double  *maxVote,
+                                           double  *denomCount,
+                                           double  *cpv);
+void getConditionalClassificationIndexPred(uint     size,
+                                           uint     rTarget,
+                                           double  *responsePtr,
+                                           double **outcomeCLS,
+                                           double  *maxVote,
+                                           double  *denomCount,
+                                           double  *cpv);
 double getClassificationIndex(uint     size,
                               uint     rTarget,
                               double  *responsePtr,
                               double  *denomCount,
                               double  *maxVote);
-double getGMeanIndex(uint    size,
-                     uint    rTarget,
-                     double *responsePtr,
-                     double *denomCount,
-                     double *maxVote);
+double getGMeanIndexGrow(uint    size,
+                         uint    rTarget,
+                         double *responsePtr,
+                         double *denomCount,
+                         double *maxVote);
+double getGMeanIndexPred(uint    size,
+                         uint    rTarget,
+                         double *responsePtr,
+                         double *denomCount,
+                         double *maxVote);
 void getMaxVote(uint     size,
                 uint     rTarget,
                 double **outcomeCLS,
